@@ -29,9 +29,12 @@ module FILEUTILS
   def cd( dn )
     if iterator? then
       pre = Dir.pwd
-      Dir.chdir dn
-      yield
-      Dir.chdir pre
+      begin
+        Dir.chdir dn
+        yield
+      ensure
+        Dir.chdir pre
+      end
     else
       Dir.chdir dn
     end
@@ -46,7 +49,7 @@ module FILEUTILS
 
   def is_older?( old, new )
     return true unless exist? old
-    File.ctime(new) <= File.ctime(old)
+    File.ctime(new) >= File.ctime(old)
   end
 
 
@@ -69,13 +72,14 @@ module FILEUTILS
 
   def ln_s( old, new )
     dest = fu_make_dest_fname( old, new )
-    File.symlink old, new
+    File.symlink old, dest
   end
 
 
   def cp( from, to )
     dest = fu_make_dest_fname( from, to )
 
+    mod = File.stat( from ).mode
     rf = File.open( from )     ; rf.binmode
     wf = File.open( dest, 'w' ); wf.binmode
     s = nil
@@ -85,6 +89,7 @@ module FILEUTILS
         wf.syswrite s
       end
     rescue EOFError
+      File.chmod mod, dest
     ensure
       rf.close
       wf.close
@@ -94,13 +99,10 @@ module FILEUTILS
   def cp_r( from, to )
     if directory? from then
       from = File.expand_path(from)
-			to = File.expand_path(to)
-			unless File.directory? to then
-			  if File.exist? to then
-				  rm_f to
-				end
-			  mkdir to
-			end
+      to = File.expand_path(to)
+      unless directory? to then
+        mkdir to
+      end
       do_cp_r from, '.', to
     else
       cp from, to
